/*
 * main.c
 */


/* Standard header files */
#include <stdio.h>
#include <string.h>
#ifndef ESIX
#include <stdlib.h>
#endif

/* Xlib and Xt Intrinsics */
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>

/* Motif Widget Set */
#include <Xm/Xm.h>
#include <Xm/DialogS.h>

/* Motif Resource Manager */
#include <Mrm/MrmAppl.h>
#include <Mrm/MrmPublic.h>

/* Local header files */
#include <defs.h>

#define APP_NAME "PatientTrack"
#define APP_CLASS "PATIENT_TRACK"

XtAppContext            context;
Display*                dpy;
Widget                  toplevelWidget;
MrmHierarchy 		hierarchy_id;
MrmType 		dummy_class;

#define NUM_UID_FILES   5
char 			*uid_files[] = {	"main.uid",
						"util.uid",
						"pat_open.uid",
						"pat_new.uid",
						"img_open.uid"
};

/* callback declaration - should move into a separate file */

extern void exit_button_cb ();
extern void exit_dialog_ok_cb ();
extern void register_widget_cb ();
extern void drawing_area_expose_cb ();
extern void patient_open_cb ();
extern void patient_new_cb ();
extern void display_or_update_cb ();
extern void patient_selection_ok ();
extern void patient_selection_cancel ();
extern void patient_selection_create ();
extern void patient_new_ok ();
extern void patient_new_cancel ();
extern void image_open_cb ();
extern void image_selection_ok ();
extern void image_selection_cancel ();
extern void image_selection_create ();
extern void edit_selection_cb ();

static 			MRMRegisterArg cb_names_list[] =
{
    {"exit_button_cb",		(caddr_t) exit_button_cb},
    {"exit_dialog_ok_cb",	(caddr_t) exit_dialog_ok_cb},
    {"edit_selection_cb",	(caddr_t) edit_selection_cb},
    {"register_widget_cb",	(caddr_t) register_widget_cb},
    {"drawing_area_expose_cb",	(caddr_t) drawing_area_expose_cb},
    {"patient_open_cb",		(caddr_t) patient_open_cb},
    {"patient_new_cb",		(caddr_t) patient_new_cb},
    {"display_or_update_cb",	(caddr_t) display_or_update_cb},
    {"patient_selection_ok",	(caddr_t) patient_selection_ok},
    {"patient_selection_cancel",(caddr_t) patient_selection_cancel},
    {"patient_selection_create",(caddr_t) patient_selection_create},
    {"patient_new_ok",		(caddr_t) patient_new_ok},
    {"patient_new_cancel",	(caddr_t) patient_new_cancel},
    {"image_open_cb",		(caddr_t) image_open_cb},
    {"image_selection_ok",	(caddr_t) image_selection_ok},
    {"image_selection_cancel",	(caddr_t) image_selection_cancel},
    {"image_selection_create",	(caddr_t) image_selection_create},
    {NULL,			(caddr_t) NULL}
};


/*
 * Global Variables
 */

static int DisplayOrUpdate = K_display;
static Pixmap pixmap = NULL;
static Dimension width = 400;
static Dimension height = 400;

int InitializeMotif (int argc, char **argv)
{
     char *new_name, *tmp;
     Widget mainWindow = NULL;
     int i;
     Arg args[8];

     /* initialize X Toolkit */
     toplevelWidget = XtVaAppInitialize (&context, APP_CLASS, NULL, 0, &argc,
                                         argv, NULL,
                                         XmNallowShellResize, TRUE, NULL);
     /* initialize Motif resource manager */
     MrmInitialize();

     /* load uid user interface modules */
     hierarchy_id = 0;
     if (MrmOpenHierarchy((MrmCount)(NUM_UID_FILES),
                      uid_files, 0, &hierarchy_id) != MrmSUCCESS) {
         fprintf (stderr, "can't open Mrm Hierarchy");
     }

     if (MrmRegisterNames(cb_names_list, 
		      (sizeof(cb_names_list)/sizeof(MRMRegisterArg))-1)
	 != MrmSUCCESS) {
         fprintf (stderr, "can't register Mrm Names");
	 exit (2);
       }


     XmRegisterConverters();

     MrmFetchWidget(hierarchy_id, "MainWindow", toplevelWidget,
                    &mainWindow, &dummy_class);

     XtManageChild(mainWindow);
     XtRealizeWidget(toplevelWidget);

     return WSL_SUCCESS;
} /* end InitializeMotif */




/*------------------------------------------------------------------------*/
/* Function Name:	main
/* 
/* Arguments:		argc and argv command line parameters.
/*
/* Return Value:	None.
/*
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

main (argc, argv)
     int argc;
     char *argv[];
{
#ifdef SHARED_LIB
    extern void _XtInherit();
    __XmInitializeShlib(_XtInherit);
#endif

    /* init Motif and Xt, bring up UIL */
    if (InitializeMotif (argc, argv) == WSL_SUCCESS)
      XtAppMainLoop (context);

    return WSL_FAILURE;
} /* end main */



void exit_button_cb (w, client_data, call_data)
     Widget w;
     caddr_t client_data;	
     XmAnyCallbackStruct *call_data;
{
    static Widget widget_id = (Widget)0;
    MrmType dummy_class;
    int arg_cnt = 0;
    Arg args[2];
    XmString title_string;

    title_string = XmStringCreateSimple ("Exit");
    XtSetArg (args[arg_cnt], XmNdialogTitle, title_string); ++arg_cnt;

    MrmFetchWidgetOverride (hierarchy_id, "ConfirmExitDialog",
			    toplevelWidget,
			    NULL,	/* don't override widget name */
			    args,
			    arg_cnt,
			    &widget_id,
			    &dummy_class);

    XtManageChild (widget_id);

    XmStringFree (title_string);
} /* end exit_button_cb */



void exit_dialog_ok_cb (w, client_data, call_data)
     Widget w;
     caddr_t client_data;
     XmAnyCallbackStruct* call_data;
{
    MrmCloseHierarchy(hierarchy_id);
    XtDestroyWidget (toplevelWidget);
    
    exit (0);
} /* end exit_dialog_ok_cb */



void register_widget_cb (w, widget_name, call_data)
     Widget w;
     int* widget_name;
     XmAnyCallbackStruct* call_data;
{
    WslStoreWidget (w, *widget_name, 0);
} /* end register_widget_cb */



void drawing_area_expose_cb (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmDrawingAreaCallbackStruct* call_data;
{
    if (pixmap)
      XCopyArea (XtDisplay(w), pixmap, call_data->window,
		 DefaultGC(XtDisplay(w),
			   DefaultScreen(XtDisplay(w))),
		 call_data->event->xexpose.x,
		 call_data->event->xexpose.y,
		 call_data->event->xexpose.width,
		 call_data->event->xexpose.height,
		 call_data->event->xexpose.x,
		 call_data->event->xexpose.y);
    
} /* end drawing_area_expose_cb */



void patient_open_cb (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmPushButtonCallbackStruct* call_data;
{
    String index = "PatientOpenWindow";
    String title = "Open Patient";
    Widget shell_widget; 
    static Widget widget_id = NULL;
    Arg args[8];
    int n = 0;

    /* create a shell and insert the proper widget */
    XtSetArg (args[n], XmNx, 100); n++;
    XtSetArg (args[n], XmNy, 150); n++;
    XtSetArg (args[n], XmNallowShellResize, TRUE); n++;

    shell_widget = XmCreateDialogShell (toplevelWidget,
					title,
					NULL, 0);

    WslStoreWidget (shell_widget, K_open_patient_widget, 0);

    if (MrmFetchWidget (hierarchy_id, index,
			shell_widget,
			&widget_id,
			&dummy_class) != MrmSUCCESS)
      return;

    XtManageChild (widget_id);
    XtRealizeWidget (shell_widget);
    
} /* end patient_open_cb */



void display_or_update_cb (w, whichButton, call_data)
     Widget w;
     int *whichButton;
     XmToggleButtonCallbackStruct* call_data;
{
    if (*whichButton == K_display && call_data->set == TRUE)
      DisplayOrUpdate = K_display;

    if (*whichButton == K_update && call_data->set == TRUE)
      DisplayOrUpdate = K_update;
    
} /* end display_or_update_cb */



void patient_selection_create (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmSelectionBoxCallbackStruct* call_data;
{
    Arg args[3];
    XmStringTable items;	/* patient, i.e. list items */
    int count = 0;		/* number of patient, i.e. list items */
    extern char* malloc ();

    /*
     * read patient names from database
     */
    count = 3;
    items = (XmStringTable) malloc (sizeof(XmString) * count);
    items[0] = XmStringCreate ("Emily Headache", XmSTRING_DEFAULT_CHARSET);
    items[1] = XmStringCreate ("Stuart Soreback", XmSTRING_DEFAULT_CHARSET);
    items[2] = XmStringCreate ("Todd Brokenleg", XmSTRING_DEFAULT_CHARSET);

    XtSetArg (args[0], XmNlistItems, items);
    XtSetArg (args[1], XmNlistItemCount, count);
    XtSetValues (w, args, 2);

    DisplayOrUpdate = K_display;
} /* end patient_selection_create */


void patient_selection_ok (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmSelectionBoxCallbackStruct* call_data;
{
    Widget shell_widget;
    Widget text_widget;
    String value;

    shell_widget = WslRetrieveWidget (K_open_patient_widget, 0);
    XtDestroyWidget (shell_widget);
    WslDeleteWidget (K_open_patient_widget, 0);

    printf ("%d\n", DisplayOrUpdate);

    if (DisplayOrUpdate == K_display) {
	/*
	 * retrieve patient data and put into text fields
	 */
	text_widget = WslRetrieveWidget (K_patient_name_text, 0);
	XmStringGetLtoR (call_data->value, XmSTRING_DEFAULT_CHARSET, &value);
	XmTextSetString (text_widget, value);
	XtFree (value);

	text_widget = WslRetrieveWidget (K_patient_date_text, 0);
	XmTextSetString (text_widget, "May 22, 1991");
    } else {
	/*
	 * pop up data entry sheet
	 */
	patient_new_cb (w, NULL, NULL);
    }
} /* end patient_selection_ok */



void patient_selection_cancel (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmPushButtonCallbackStruct* call_data;
{
    Widget shell_widget;

    shell_widget = WslRetrieveWidget (K_open_patient_widget, 0);
    XtDestroyWidget (shell_widget);
    WslDeleteWidget (K_open_patient_widget, 0);
} /* end patient_selection_cancel */


void patient_new_cb (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmPushButtonCallbackStruct* call_data;
{
    String index = "PatientNewWindow";
    String title = "New Patient";
    Widget shell_widget; 
    static Widget widget_id = NULL;
    Arg args[8];
    int n = 0;

    /* create a shell and insert the proper widget */
    XtSetArg (args[n], XmNx, 100); n++;
    XtSetArg (args[n], XmNy, 150); n++;
    XtSetArg (args[n], XmNallowShellResize, TRUE); n++;

    shell_widget = XmCreateDialogShell (toplevelWidget,
					title,
					NULL, 0);

    WslStoreWidget (shell_widget, K_new_patient_widget, 0);

    if (MrmFetchWidget (hierarchy_id, index,
			shell_widget,
			&widget_id,
			&dummy_class) != MrmSUCCESS)
      return;

    XtManageChild (widget_id);
    XtRealizeWidget (shell_widget);
    
} /* end patient_new_cb */


void patient_new_cancel (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmPushButtonCallbackStruct* call_data;
{
    Widget shell_widget;

    shell_widget = WslRetrieveWidget (K_new_patient_widget, 0);
    XtDestroyWidget (shell_widget);
    WslDeleteWidget (K_new_patient_widget, 0);
} /* end patient_new_cancel */


void patient_new_ok (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmPushButtonCallbackStruct* call_data;
{
    Widget shell_widget;

    shell_widget = WslRetrieveWidget (K_new_patient_widget, 0);
    XtDestroyWidget (shell_widget);
    WslDeleteWidget (K_new_patient_widget, 0);
} /* end patient_new_ok */


void image_open_cb (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmPushButtonCallbackStruct* call_data;
{
    String index = "ImageOpenWindow";
    String title = "Open Image";
    Widget shell_widget; 
    static Widget widget_id = NULL;
    Arg args[8];
    int n = 0;

    /* create a shell and insert the proper widget */
    XtSetArg (args[n], XmNx, 100); n++;
    XtSetArg (args[n], XmNy, 150); n++;
    XtSetArg (args[n], XmNallowShellResize, TRUE); n++;
    shell_widget = XtAppCreateShell( title, APP_CLASS,
				    topLevelShellWidgetClass,
				    XtDisplay(w),
				    args, n);
    WslStoreWidget (shell_widget, K_open_image_widget, 0);

    if (MrmFetchWidget (hierarchy_id, index,
			shell_widget,
			&widget_id,
			&dummy_class) != MrmSUCCESS)
      return;

    XtManageChild (widget_id);
    XtRealizeWidget (shell_widget);
    
} /* end image_open_cb */


void image_selection_cancel (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmPushButtonCallbackStruct* call_data;
{
    Widget shell_widget;

    shell_widget = WslRetrieveWidget (K_open_image_widget, 0);
    XtDestroyWidget (shell_widget);
    WslDeleteWidget (K_open_image_widget, 0);
} /* end image_selection_cancel */


void image_selection_ok (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmPushButtonCallbackStruct* call_data;
{
    Widget shell_widget, drawing_area;

    shell_widget = WslRetrieveWidget (K_open_image_widget, 0);
    XtDestroyWidget (shell_widget);
    WslDeleteWidget (K_open_image_widget, 0);

    /*
     * 1. Read the image file from disk
     * 2. Create a pixmap and put the image into the pixmap
     * 3. Copy the pixmap to the DrawingArea using XCopyArea
     */

    drawing_area = WslRetrieveWidget (K_drawing_area, 0);
    if (pixmap != NULL) XFreePixmap (XtDisplay(w), pixmap);
    pixmap = XCreatePixmap (XtDisplay(w), XtWindow(drawing_area),
			    width, height, 
			    DefaultDepth(XtDisplay(w),
					 DefaultScreen(XtDisplay(w))));
    XCopyArea (XtDisplay(w), pixmap, XtWindow(drawing_area),
	       DefaultGC(XtDisplay(w),
			 DefaultScreen(XtDisplay(w))),
	       0, 0, width, height, 0, 0);
    
} /* end image_selection_ok */


void image_selection_create (w, no_data, call_data)
     Widget w;
     caddr_t no_data;
     XmFileSelectionBoxCallbackStruct* call_data;
{
    /* specify dir mask, etc. */
} /* end image_selection_create */



void edit_selection_cb (w, whichButton, call_data)
     Widget w;
     int *whichButton;
     XmPushButtonCallbackStruct* call_data;
{
    static Widget text_field = 0;

    if (call_data->reason == XmCR_GAIN_PRIMARY) {
	printf ("gained primary\n");
	text_field = w;
    } else {
	switch (*whichButton) {
	  case 1:
	    printf ("Copy\n");
	    XmTextFieldCopy (text_field, call_data->event->xbutton.time);
	    break;
	  case 2:
	    printf ("Cut\n");
	    XmTextFieldCut(text_field, call_data->event->xbutton.time);
	    break;
	  case 3:
	    printf ("Paste\n");
	    /* XmTextFieldPaste (paste_text_field); */
	    break;
	}	
    }
} /* end edit_selection_cb */
